(
  ($) ->
    class SearchSelector
      templates: {
        ui: '''
        <div class="ss-dropdown ss-hide">
          <div>
            <input type="text" placeholder="начните вводить..." class="ss-filter-field">
          </div>
          <ul class="ss-items">
          </ul>
        </div>
        '''

        item: '''
        <li>
          <a href="#" class="ss-item-handler" data-id="{id}">{name}</a>
        </li>
        '''
      }

      triggers:
        opened: 'ss-opened'
        closed: 'ss-closed'
        change: 'ss-change'
        keyup:   'ss-keyup'

      css:
        hide: 'ss-hide'

      selectors:
        handler:      '.ss-handler'
        itemHandler: '.ss-item-handler'
        dropDown:    '.ss-dropdown'
        items:       '.ss-dropdown .ss-items'
        item:        '.ss-dropdown .ss-items .ss-item-handler'
        filterField: '.ss-filter-field'

      bindings: {
        'click handler': ->
          if @isOpen
            @close()
          else
            @open()

        'click item': (e) ->
          e.preventDefault()
          cityId = $(e.currentTarget).data('id')
          @select(cityId)
          @close()

        'keyup filterField': (e) ->
          @trigger('keyup', e.currentTarget)
      }

      find: (selector) -> @el.find(selector)
      trigger: (triggerName, options = null) ->
        @handler.trigger(@triggers[triggerName], options)

      constructor: (@options = {}) ->
        @el = $(options['el'])
        throw 'Anchor element not found' unless @el.length
        @selected = null
        @selected_caption = null

        @el.append($(@template('ui')))
        @handler = @find(@selectors.handler).eq(0)
        for event_selector, callback of @bindings
          do (event_selector, callback) =>
            [event, selector] = event_selector.split(' ')
            # Заменяем на css селектор
            selector = @selectors[selector]
            @el.on event, selector, =>
              callback.apply(@, arguments) if typeof callback == 'function' and event and selector
        @init()

      init: ->
        @$dropDown = @find(@selectors.dropDown)
        @$items = @find(@selectors.items)
        @isOpen = false
        @items = []
        @setItems(@options['items']) if @options['items']?
        @select(@options['value']) if @options['value']?

      open: ->
        offset = @handler.offset()
        @$dropDown.css(top: offset.top + parseInt(@handler.outerHeight(true)))
        @$dropDown.css(left: offset.left)
        @$dropDown.removeClass(@css.hide)
        @isOpen = true
        @trigger('opened')

      close: ->
        @$dropDown.addClass(@css.hide)
        @isOpen = false
        @trigger('closed')

      drawDropDown: ->
        @$items.empty()
        for item of @items
          [key, val] =
            if @items instanceof Array
              [item, item]
            else
              [item, @items[item]]
          $item = $(@template('item', id: key, name: val))
          @$items.append($item)

      template: (name, params = {}) ->
        template = @templates[name]
        for key, val of params
          template = template.replace(new RegExp("\\{#{key}\\}", 'gi'), val)
        template

      select: (item) ->
        if @items instanceof Array
          if item in @items
            @selected = item
            @selected_caption = item
            @handler.html(item)
            @trigger('change', [@val(), @text()])
        else
          if item.toString() in Object.keys(@items)
            @selected = item
            @selected_caption = @items[item]
            @handler.html(@text())
            @trigger('change', [@val(), @text()])

      val: -> @selected

      text: -> @selected_caption

      setItems: (items) ->
        @items = items
        @drawDropDown()

    jQuery.fn.searchSelector = (options) ->
      $el = $(this)
      if typeof options == 'string'
        searchSelector = $el.data('search-selector')
        switch options
          when 'val'
            searchSelector.selected
          when 'text'
            searchSelector.text()
          when 'searchSelector'
            searchSelector
      else
        $ss = $('<span class="ss-element"></span>')
        $el.before($ss).detach()
        $ss.append($el)
        $el.addClass('ss-handler')
        searchSelector = new SearchSelector($.extend(options, el: $ss))
        $ss.data('search-selector', searchSelector)
        $el.data('search-selector', searchSelector)

        $(document).on 'click', (e) ->
          searchSelector.close() if $(e.target).closest('.ss-element').data('search-selector') != searchSelector

)(jQuery)







